@@ -17,11 +17,7 @@ pub struct Rotation(Vector3<f32>);
 #[derive(Copy, Clone)]
 pub struct Velocity(Vector3<f32>);
 
-pub struct Benchmark<S2, S4>
-where
-    S2: WorldStorage<Component = Position>,
-    S4: WorldStorage<Component = Velocity>,
-{
+pub struct Benchmark<S2, S4> {
     ps: S2, vs: S4,
 }
 
@@ -47,7 +43,7 @@ where
     }
 
     pub fn run(&mut self) {
-        for (_, velocity, position) in (&self.vs, &mut self.ps).join() {
+        for (velocity, position) in (&self.vs, &mut self.ps).join() {
             position.0 += velocity.0;
         }
     }
